"use strict";
/*!
   Copyright 2019 Ron Buckton

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
var _a, _b;
Object.defineProperty(exports, "__esModule", { value: true });
exports.AsyncMutex = void 0;
const async_lockable_1 = require("@esfx/async-lockable");
const async_waitqueue_1 = require("@esfx/async-waitqueue");
const cancelable_1 = require("@esfx/cancelable");
const disposable_1 = require("@esfx/disposable");
let lock;
let unlock;
let ownsLock;
/**
 * An async coordination primitive used to coordinate access to a protected resource.
 */
class AsyncMutex {
    constructor() {
        this._waiters = new async_waitqueue_1.WaitQueue();
        // #endregion AsyncLockable
    }
    /**
     * Indicates whether the lock has been taken.
     */
    get isLocked() {
        return this._handle !== undefined;
    }
    /**
     * Asynchronously waits for the lock to become available and then takes the lock.
     * @param cancelable A `Cancelable` used to cancel the pending request.
     */
    async lock(cancelable) {
        const handle = new AsyncMutexLockHandle(this);
        await handle.lock(cancelable);
        return handle;
    }
    /**
     * Synchronously tries to take a lock.
     */
    tryLock() {
        if (!this._handle) {
            this._handle = new AsyncMutexLockHandle(this);
            return true;
        }
        return false;
    }
    /**
     * Releases the lock.
     */
    unlock() {
        if (this._handle) {
            this._unlock(this._handle);
            return true;
        }
        return false;
    }
    async _lock(handle, cancelable) {
        var _c;
        if (!(cancelable === undefined) && !cancelable_1.Cancelable.hasInstance(cancelable))
            throw new TypeError("Cancelable expectd: cancelable");
        const signal = cancelable === null || cancelable === void 0 ? void 0 : cancelable[cancelable_1.Cancelable.cancelSignal]();
        if (signal === null || signal === void 0 ? void 0 : signal.signaled)
            throw (_c = signal.reason) !== null && _c !== void 0 ? _c : new cancelable_1.CancelError();
        if (this._handle === handle) {
            throw new Error("Lock already taken.");
        }
        if (this._handle) {
            await this._waiters.wait(cancelable);
            if (this._handle === handle) {
                throw new Error("Lock already taken.");
            }
        }
        this._handle = handle;
    }
    _unlock(handle) {
        if (this._handle !== handle) {
            throw new Error("Lock already released.");
        }
        if (!this._waiters.resolveOne()) {
            this._handle = undefined;
        }
    }
    // #region AsyncLockable
    [async_lockable_1.AsyncLockable.lock](cancelable) {
        return this.lock(cancelable);
    }
    [async_lockable_1.AsyncLockable.unlock]() {
        this.unlock();
    }
}
exports.AsyncMutex = AsyncMutex;
_a = AsyncMutex;
(() => {
    Object.defineProperty(_a.prototype, Symbol.toStringTag, { configurable: true, value: "AsyncMutex" });
    lock = (mutex, handle, cancelable) => mutex._lock(handle, cancelable);
    unlock = (mutex, handle) => mutex._unlock(handle);
    ownsLock = (mutex, handle) => mutex._handle === handle;
})();
class AsyncMutexLockHandle {
    constructor(mutex) {
        this._mutex = mutex;
    }
    get mutex() {
        return this._mutex;
    }
    get ownsLock() {
        return ownsLock(this._mutex, this);
    }
    async lock(cancelable) {
        await lock(this._mutex, this, cancelable);
        return this;
    }
    unlock() {
        unlock(this._mutex, this);
    }
    [async_lockable_1.AsyncLockable.lock](cancelable) {
        return this.lock(cancelable);
    }
    [async_lockable_1.AsyncLockable.unlock]() {
        return this.unlock();
    }
    [disposable_1.Disposable.dispose]() {
        if (this.ownsLock) {
            this.unlock();
        }
    }
}
_b = AsyncMutexLockHandle;
(() => {
    Object.defineProperty(_b, "constructor", { ...Object.getOwnPropertyDescriptor(_b, "constructor"), value: Object });
    Object.defineProperty(_b.prototype, Symbol.toStringTag, { configurable: true, value: "AsyncMutexLockHandle" });
})();
//# sourceMappingURL=index.js.map